import hashlib
import sqlite3
import time
import logging

log = logging.getLogger('acss')

def _dict_factory(cursor, row):
    d = {}
    for idx, col in enumerate(cursor.description):
        d[col[0]] = row[idx]
    return d

def best_lap_id(track, driver, car):
    return hashlib.sha224("%s-%s-%s" % (driver, track,
        car)).hexdigest()

class DB(object):
    def __init__(self, db_filename):
        self.db_filename = db_filename
        self.dbc = self.get_sqlite_dbconnection()
        self.prepare_db()

    def get_sqlite_dbconnection(self):
        dbc = sqlite3.connect(self.db_filename)
        dbc.row_factory = _dict_factory
        return dbc

    def prepare_db(self):
        c = self.dbc.cursor()

        try:
            c.execute('''CREATE TABLE best_laps
                         (id text,
                        driver_guid text,
                        driver_name text,
                        track_name text,
                        car_name text,
                        ballast_kg int,
                        last_update text,
                        best_lap bigint)''')
        except sqlite3.OperationalError:
            log.info('Table best_laps already created')

    def get_tracks(self):
        c = self.dbc.cursor()
        query = """select distinct track_name
            from best_laps
            order by last_update desc"""
        c.execute(query)
        return [t['track_name'] for t in c.fetchall()]

    def get_best_laps(self, track_name, limit=None, car_names=None):
        query = """select driver_name, track_name, best_lap, car_name
                    from best_laps
                    where """
        where = [ ('track_name', track_name), ('car_name', car_names) ]
        fields, values = [], []
        infields, invalues = [], []

        for field, value in where:
            if value is None:
                continue
            if isinstance(value, list):
                infields.append(field)
                invalues.append(value)
            else:
                fields.append(field)
                values.append(value)
        query += ' and '.join(["%s = ?" % (f,) for f in fields])

        for idx, field in enumerate(infields):
            vals = invalues[idx]
            query += " and %s in (%s)" % (field, ', '.join('?'*len(vals)))
            values.extend(vals)

        query += " order by track_name, car_name, best_lap asc"
        
        if limit is not None:
            query += " limit %d" % (limit,)

        c = self.dbc.cursor()
        c.execute(query, values)
        data = c.fetchall()

        c.execute(
            """select last_update
            from best_laps
            order by last_update desc limit 1""")
        last_update = c.fetchone()
        if last_update:
            last_update = last_update['last_update']

        return {'data': data,
            'last_update': last_update}

    def get_best_lap_by_id(self, id_):
        c = self.dbc.cursor()
        c.execute(
            "select * from best_laps where id = ?", (id_,))
        return c.fetchone()

    def get_best_lap(self, driver_guid, track, car):
        id_ = best_lap_id(track, driver_guid, car)
        c = self.dbc.cursor()
        c.execute(
            "select * from best_laps where id = ?", (id_,))
        return c.fetchone()

    def insert_best_lap(self, driver_guid, driver, track, car, ballast, best_lap):
        id_ = best_lap_id(track, driver_guid, car)
        c = self.dbc.cursor()
        c.execute("""insert into best_laps(id, driver_guid,
                driver_name, track_name, car_name, ballast_kg,
                best_lap, last_update)
            values (?,?,?,?,?,?,?,?)""",
            (id_, driver_guid, driver, track, car, ballast, best_lap,
                time.time()))
        self.dbc.commit()

    def delete_best_lap(self, driver_guid, track, car):
        id_ = best_lap_id(track, driver_guid, car)
        c = self.dbc.cursor()
        c.execute("""delete from best_laps where id = ?""",
            (id_,))
        self.dbc.commit()



import sqlite3
import sys
import json

from waitress import serve

from pyramid.config import Configurator
from pyramid.response import Response

from acss.db import DB
from acss.client import Server

__VERSION__ = 0.1


def api_tracks(request):
    return request.db().get_tracks()

def api_track_bestlaps(request):
    return request.db().get_best_laps(request.matchdict['track_name'])

def api_track_bestlaps_with_cars(request):
    return request.db().get_best_laps(request.matchdict['track_name'],
        car_names=request.matchdict['car_names'].split(','))

def api_server_info(request):
    s = request.registry.settings
    try:
        return {'success': True,
            'data': Server.get_info(s['server_host'], s['server_port'])}
    except:
        return {'success': False}

def run():
    if len(sys.argv) != 2:
        sys.stderr.write("Usage: %s <config_file>\n" % (sys.argv[0],))
        sys.exit(1)

    settings = {'db_filename': './acss.db', 'server_host': 'localhost',
        'server_port': 8081}
    settings.update(json.loads(open(sys.argv[1]).read()))

    config = Configurator(settings=settings)

    def add_db(request, reify=True):
        return DB(request.registry.settings.get('db_filename'))

    config.add_request_method(add_db, 'db')

    config.add_route('api_tracks', '/api/tracks')
    config.add_route('api_track_bestlaps', '/api/tracks/{track_name}/bestlaps')
    config.add_route('api_track_bestlaps_with_cars', '/api/tracks/{track_name}/{car_names}/bestlaps')
    config.add_route('api_server_info', '/api/server_info')

    config.add_view(api_tracks, route_name='api_tracks', renderer='json')
    config.add_view(api_track_bestlaps, route_name='api_track_bestlaps',
        renderer='json')
    config.add_view(api_track_bestlaps_with_cars,
        route_name='api_track_bestlaps_with_cars', renderer='json')
    config.add_view(api_server_info, route_name='api_server_info',
        renderer='json')

    app = config.make_wsgi_app()
    serve(app, host='0.0.0.0', port=8001)



import urllib2
import json

class Client(object):
    def __init__(self):
        pass

    @classmethod
    def get_entry(cls, host, port=8081, timeout=3.0):
        """
        Returns HTML data containing drivers on the servers, their guid, cars.
        """
        return urllib2.urlopen("http://%s:%d/ENTRY" % (host, port),
            timeout=timeout).read()

    @classmethod
    def get_info(cls, host, port=8081, timeout=3.0):
        """
        Fetchs and returns a json object containing server information.
        """
        return json.loads(urllib2.urlopen("http://%s:%d/INFO" % (host, port),
            timeout=timeout).read())



import os

from setuptools import setup, find_packages

from acss import VERSION

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.md')) as f:
    README = f.read()
with open(os.path.join(here, 'requirements.txt')) as f:
    requires = f.read().splitlines()

setup(name='acss',
      version=VERSION,
      description='acss',
      long_description=README + '\n',
      classifiers=[
        "Programming Language :: Python",
        "Framework :: Pyramid",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        ],
      author='Joao Coutinho',
      author_email='me at joaoubaldo.com',
      url='https://b.joaoubaldo.com',
      keywords='web assetto-corsa dedicated server stats statistics timing',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=requires,
      tests_require=requires,
      test_suite="acss",
      entry_points={
        'console_scripts':
            ['acss_cron = acss.cli.cron:run',
            'acssd = acss.cli.wsgi_service:run']
        }
      )



import os
import sys
import re
import logging
import json

from acss.db import DB

logging.basicConfig(level=logging.DEBUG)
log = logging.getLogger('acss')

def load_results_from_directory(in_dir, out_db):
    db = DB(out_db)
    in_files = os.listdir(in_dir)

    for filename in in_files:
        file_regex = "^(\d\d\d\d)_(\d\d*)_(\d\d*)_(\d\d*)_(\d\d*)_(.*)\.json$"
        file_match = re.match(file_regex, filename)

        if file_match is None:
            continue

        log.info("Picking file %s" % (filename,))

        year, month, day, hour, minute, type_ = file_match.groups()

        f = open(os.path.join(in_dir, filename), 'r')
        data = json.loads(f.read())
        f.close()

        for driver in data['Result']:
            driver_row = db.get_best_lap(
                driver['DriverGuid'], data['TrackName'], driver['CarModel'])

            def insert_driver():
                db.insert_best_lap(
                    driver['DriverGuid'], driver['DriverName'],
                    data['TrackName'], driver['CarModel'],
                    driver['BallastKG'], driver['BestLap'])

            def delete_driver():
                db.delete_best_lap(
                    driver['DriverGuid'], data['TrackName'], driver['CarModel'])

            if driver['BestLap'] >= 999999:
                continue

            if not driver_row:
                log.info("New id for driver %s" % (driver['DriverName'],))
                insert_driver()
            else:
                if driver['BestLap'] < driver_row['best_lap']:
                    log.info("Driver %s has a bestlap new [%s] old [%s]" % (
                        driver_row['driver_name'], driver, driver_row))
                    delete_driver()
                    insert_driver()


def run():
    if len(sys.argv) != 3:
        sys.stderr.write("Usage: %s <input folder> <sqlite file>" % (
            sys.argv[0],))
        sys.exit(1)

    load_results_from_directory(sys.argv[1], sys.argv[2])
    sys.exit(0)
